@@ -12,8 +12,8 @@ EV - perl interface to libev, a high performance full-featured event loop
       warn "is called after 2s";
    };
    
-   my $w = EV::timer 2, 2, sub {
-      warn "is called roughly every 2s (repeat = 2)";
+   my $w = EV::timer 0, 3, sub {
+      warn "is called as soon as possible, then every 3s";
    };
    
    undef $w; # destroy event watcher again
@@ -37,7 +37,7 @@ EV - perl interface to libev, a high performance full-featured event loop
    
    # CHILD/PID STATUS CHANGES
   
-   my $w = EV::child 666, 0, sub {
+   my $w = EV::child $pid, 0, sub {
       my ($w, $revents) = @_;
       my $status = $w->rstatus;
    };
@@ -121,7 +121,7 @@ package EV;
 use common::sense;
 
 BEGIN {
-   our $VERSION = '4.30';
+   our $VERSION = '4.35';
    use XSLoader;
    local $^W = 0; # avoid spurious warning
    XSLoader::load "EV", $VERSION;
@@ -177,7 +177,7 @@ or locally-installed as F<EV::libev> manpage) for more info.
 The loop will automatically be destroyed when it is no longer referenced
 by any watcher and the loop object goes out of scope.
 
-If you are not embedding the loop, then Using C<EV::FLAG_FORKCHECK>
+If you are not embedding the loop, then using C<EV::FLAG_FORKCHECK>
 is recommended, as only the default event loop is protected by this
 module. If you I<are> embedding this loop in the default loop, this is not
 necessary, as C<EV::embed> automatically does the right thing on fork.
@@ -648,6 +648,14 @@ value.
 
 Calculates and returns the remaining time till the timer will fire.
 
+=item $repeat = $w->repeat
+
+=item $old_repeat = $w->repeat ($new_repeat)
+
+Returns the current value of the repeat attribute and optionally sets a
+new one. Setting the new one will not restart the watcher - if the watcher
+is active, the new repeat value is used whenever it expires next.
+
 =back
 
 
@@ -750,6 +758,26 @@ Simply stops and starts the watcher again.
 
 Return the time that the watcher is expected to trigger next.
 
+=item $offset = $w->offset
+
+=item $old_offset = $w->offset ($new_offset)
+
+Returns the current value of the offset attribute and optionally sets a
+new one. Setting the new one will not restart the watcher - if the watcher
+is active, the new offset value is used whenever it expires next.
+
+=item $interval = $w->interval
+
+=item $old_interval = $w->interval ($new_interval)
+
+See above, for the interval attribute.
+
+=item $reschedule_cb = $w->reschedule_cb
+
+=item $old_reschedule_cb = $w->reschedule_cb ($new_reschedule_cb)
+
+See above, for the reschedule callback.
+
 =back
 
 
@@ -847,6 +875,17 @@ in perlfunc).
 Return the pid of the awaited child (useful when you have installed a
 watcher for all pids).
 
+=item EV::Child::reinit [EXPERIMENTAL]
+
+Internally, libev installs a signal handler for C<SIGCHLD>. Unfortunately,
+a lot of Perl code does soemthing like C<< local $SIG{CHLD} >>, which,
+unfortunately, is broken and will not restore the signal handler.
+
+If this has happened, you can call this function to stop/rrestart the
+internal libev watcher, which will reset the signal handler.
+
+Note that this is an experimental function, whose interface might change.
+
 =back
 
 
